module alu(di1, di2, dout, alu_ops, apsr);
   input [15:0] di1, di2;		// data input
   output [15:0] dout;			// data output
   input [2:0] 	 alu_ops;		// enable operation: [2] => add, [1] => and, [0] => not
   output [4:0]  apsr;			// arithmetic program status register
   reg [15:0] 	 dout;

   always @(alu_ops or di1 or di2)
     if (alu_ops[2])
       dout = di1 + di2;
     else if (alu_ops[1])
       dout = di1 & di2;
     else if (alu_ops[0])
       dout = ~di1;
     else	// default output di1
       dout = di1;

   assign	apsr[1] = (dout == 0);			// zero
   assign	apsr[2] = dout[15] & ~apsr[1];	// negative
   assign	apsr[0] = ~dout[15] & ~apsr[1];	// positive
   assign	apsr[4] = alu_ops[2] & di1[15] & di2[15] & ~dout[15];	// negative overflow: neg + neg = posa
   assign	apsr[3] = alu_ops[2] & ~di1[15] & ~di2[15] & dout[15];	// positive overflow: pos + pos = neg

endmodule
