module greg(rst, clk, flu, ra1, rd1, ra2, rd2, we1, wa1, wd1, we2, wa2, wd2, pc, savepc_to_R7, savepc_to_stack, loadpc_from_stack);
   input rst, flu;
   input clk;
   input we1, we2;
   input [2:0] ra1, ra2, wa1, wa2;
   input [15:0] wd1, wd2;
   output [15:0] rd1, rd2;

   input [15:0]  pc;
   input	 savepc_to_R7;
   
   input 	savepc_to_stack, loadpc_from_stack;
   wire [2:0]	real_ra2; // if push or pop, then read R6 for stack addr
   assign	real_ra2 = (savepc_to_stack | loadpc_from_stack) ? 3'b110 : ra2;

   wire [7:0] 	 we1_dw;
   wire [7:0] 	 we2_dw;
   reg [7:0] 	 we1_d;
   reg [7:0] 	 we2_d;

   wire [15:0] 	 r [7:0];
   wire [15:0]	 real_r6;
   
   assign we1_dw = we1 << wa1;
   assign we2_dw = we2 << wa2;

   always@(negedge clk)
     begin
	we1_d <= we1_dw;
	we2_d <= we2_dw;
     end

   sreg sreg_0(rst, clk, flu, r[0], , we1_d[0], wd1, we2_d[0], wd2, 1'b0);
   sreg sreg_1(rst, clk, flu, r[1], , we1_d[1], wd1, we2_d[1], wd2, 1'b0);
   sreg sreg_2(rst, clk, flu, r[2], , we1_d[2], wd1, we2_d[2], wd2, 1'b0);
   sreg sreg_3(rst, clk, flu, r[3], , we1_d[3], wd1, we2_d[3], wd2, 1'b0);
   sreg sreg_4(rst, clk, flu, r[4], , we1_d[4], wd1, we2_d[4], wd2, 1'b0);
   sreg sreg_5(rst, clk, flu, r[5], , we1_d[5], wd1, we2_d[5], wd2, 1'b0);
   sreg sreg_6(rst, clk, flu, real_r6, , we1_d[6], wd1, we2_d[6], wd2,
	savepc_to_stack | loadpc_from_stack,	// flush if stack operation
	savepc_to_stack ? real_r6 - 1 : real_r6 + 1	// save: push, load: pop
	);
   // the current R6 points to an empty cell in stack
   assign r[6] = loadpc_from_stack ? real_r6 + 1 : real_r6;
   sreg sreg_7(rst, clk, flu, r[7], , we1_d[7], wd1, we2_d[7], wd2, savepc_to_R7, pc);
   
   assign rd1 = r[ra1];
   assign rd2 = r[real_ra2];

endmodule // greg
