module stage_mem(clk, addr, rd, wd, we, apsr, indirect_stage, indirect_inst);
   input 	 clk;
   input [15:0]  addr;	// only one address
   output [15:0] rd;	// read data
   input [15:0]  wd;	// write data
   input 	 we;	// write enable
   output [4:0]  apsr;	// APSR generated by mem (may not be used)
   
   // for indirect mem ops, the first cycle's output data should be the addr of second cycle
   // Stalling pipeline has been controlled elsewhere. Here we can assume the instruction repeats for two cycles.
   input	indirect_inst;	// this instruction needs indirect memory
   input	indirect_stage;	// this stage is the second stage of indirect memory
   reg [8:0] 	indirect_buffer;// here buffers the MEM output of the last cycle

   always @(posedge clk)
		indirect_buffer <= rd[8:0];

   wire [8:0]	real_addr  /* synthesis keep */;
   wire 	real_write /* synthesis keep */;
   // first cycle real addr from ALU, second cycle from buffer
   assign real_addr = indirect_stage ? indirect_buffer : addr[8:0];
   // first cycle read, second cycle read(LDI) or write(STI)
   assign real_write = indirect_stage ? we : ~indirect_inst & we;
   
   // READ and WRITE take place at downside of CLK, giving enough time for address calculation,
   //	while prevent from reading or writing too many times because of glitches.
   memory data_mem(.clk(~clk),
   		   .addr(real_addr),
		   .rd(rd),
		   .wd(wd),
		   .we(real_write),
   		   .apsr(apsr));

endmodule // stage_mem
