module state_pack(state, pc, op, dra, sr1a, sr2a, iswb_alu, iswb_mem, imm, alu_ops, c_sr1, c_sr2,
		  sjnzp, memw, apsr_is_alu, apsr_is_mem, load_mem, arith_alu, indirect_mem, trap, exc, rti);
   input [15:0] pc;
   input [3:0] 	op;
   input [2:0] 	dra, sr1a, sr2a;
   input 	iswb_alu, iswb_mem;
   input [15:0] imm;
   input [2:0] 	alu_ops;
   input 	c_sr1, c_sr2;
   input [4:0] 	sjnzp;
   input 	memw;
   input	apsr_is_alu;
   input	apsr_is_mem;
   input	load_mem;
   input	arith_alu;
   input	indirect_mem;
   input 	trap;
   input	exc;
   input	rti;

   output [65:0] state;

   // see state_unpack.v for comments of the following fields
   assign state[15:0]  = pc;
   assign state[19:16] = op;
   assign state[22:20] = dra;
   assign state[25:23] = sr1a;
   assign state[28:26] = sr2a;
   assign state[29:29] = iswb_alu;
   assign state[30:30] = iswb_mem;
   assign state[46:31] = imm;
   assign state[49:47] = alu_ops;
   assign state[50:50] = c_sr1;
   assign state[51:51] = c_sr2;
   assign state[56:52] = sjnzp;
   assign state[57] = memw;
   assign state[58] = apsr_is_alu;
   assign state[59] = apsr_is_mem;
   assign state[60] = load_mem;
   assign state[61] = arith_alu;
   assign state[62] = indirect_mem;
   assign state[63] = trap;
   assign state[64] = exc;
   assign state[65] = rti;

endmodule // state_pack

/*
 * pc is program counter of this instruction
 * op is opcode of this instruction
 * dra is destination register address
 * sr[12]a is source register address
 * 
 * alu_ops[2:0] = { do_add, do_and, do_not }
 *
 * sjnzp means { save_pc, j-type, negative, zero, positive }
 * { save_pc }: sjnzp[4] == 1 iff instruction is j-type and need to save pc to r7
 * { j-type  }: sjnzp[3] == 1 iff instruction is j-type and is not BRx
 * {   nzp   }: sjnzp[2:0] == ir[11:9] if instruction is BRx
 * 
 */
