module state_unpack(state, pc, op, dra, sr1a, sr2a, iswb_alu, iswb_mem, imm, alu_ops, c_sr1, c_sr2,
		    sjnzp, memw, apsr_is_alu, apsr_is_mem, load_mem, arith_alu, indirect_mem, trap, exc, rti);
   input [65:0] state;

   output [15:0] pc;
   output [3:0]  op;
   output [2:0]  dra, sr1a, sr2a;
   output 	 iswb_alu, iswb_mem;
   output [15:0] imm;
   output [2:0]  alu_ops;
   output 	 c_sr1, c_sr2;
   output [4:0]  sjnzp;
   output 	 memw;
   output	 apsr_is_alu;
   output	 apsr_is_mem;
   output	 load_mem;
   output	 arith_alu;
   output	 indirect_mem;
   output 	 trap;
   output	 exc;
   output	 rti;

   assign pc = state[15:0];		// program counter
   assign op = state[19:16];		// opcode of instruction
   assign dra = state[22:20];		// destination register address
   assign sr1a = state[25:23];		// source register 1 address
   assign sr2a = state[28:26];		// source register 2 address
   assign iswb_alu = state[29:29];	// ALU output should write register
   assign iswb_mem = state[30:30];	// MEM output should write register
   assign imm = state[46:31];		// immediate
   assign alu_ops = state[49:47];	// ALU operations
   assign c_sr1 = state[50:50];		// ALU source 1: PC or SR1
   assign c_sr2 = state[51:51];		// ALU source 2: immediate or SR2
   assign sjnzp = state[56:52];		// jump conditions
   assign memw = state[57];		// memory write
   assign apsr_is_alu = state[58];	// PSR is from ALU
   assign apsr_is_mem = state[59];	// PSR is from MEM
   assign load_mem = state[60];		// the instruction needs to load from memory
   assign arith_alu = state[61];	// the instruction is R-type
   assign indirect_mem = state[62];	// the instruction needs indirect memory access
   assign trap = state[63];		// the instruction is TRAP
   assign exc = state[64];		// exception instruction
   assign rti = state[65];		// RTI instruction
   
endmodule // state_unpack
